@@ -241,7 +241,6 @@ required.
 
 ## TODO
 
-* Connection pooling using [ODPI-C Pool Functions][] (Note: [r2d2-oracle][] is available for connection pooling.)
 * [BFILEs (External LOBs)](https://www.oracle.com/pls/topic/lookup?ctx=dblatest&id=GUID-5834BC49-4053-40FF-BE39-B14342B1201E) (Note: Reading contents of BFILEs as `Vec<u8>` is supported.)
 * Scrollable cursors
 * Better Oracle object type support
@@ -260,8 +259,6 @@ Rust-oracle and ODPI-C bundled in rust-oracle are under the terms of:
 [ODPI-C installation document]: https://oracle.github.io/odpi/doc/installation.html
 [Oracle database]: https://www.oracle.com/database/index.html
 [NLS_LANG]: https://www.oracle.com/pls/topic/lookup?ctx=dblatest&id=GUID-86A29834-AE29-4BA5-8A78-E19C168B690A
-[ODPI-C Pool Functions]: https://oracle.github.io/odpi/doc/functions/dpiPool.html
-[r2d2-oracle]: https://crates.io/crates/r2d2-oracle
 */
 
 use lazy_static::lazy_static;
@@ -271,6 +268,8 @@ use std::ptr;
 use std::result;
 use std::slice;
 
+#[cfg(feature = "aq_unstable")]
+pub mod aq;
 mod batch;
 #[allow(dead_code)]
 #[allow(non_camel_case_types)]
@@ -280,10 +279,14 @@ mod batch;
 #[allow(deref_nullptr)] // ditto
 mod binding;
 mod binding_impl;
+pub mod conn;
 mod connection;
 mod error;
 pub mod io;
 pub mod oci_attr;
+pub mod pool;
+#[cfg(doctest)]
+mod procmacro;
 mod row;
 pub mod sql_type;
 mod sql_value;
@@ -315,6 +318,7 @@ pub use crate::statement::StatementBuilder;
 pub use crate::statement::StatementType;
 pub use crate::statement::StmtParam;
 pub use crate::version::Version;
+pub use oracle_procmacro::RowValue;
 
 use crate::binding::*;
 
@@ -365,12 +369,21 @@ macro_rules! define_dpi_data_with_refcount {
 // define DpiConn wrapping *mut dpiConn.
 define_dpi_data_with_refcount!(Conn);
 
+// define DpiMsgProps wrapping *mut dpiMsgProps.
+define_dpi_data_with_refcount!(MsgProps);
+
 // define DpiObjectType wrapping *mut dpiObjectType.
 define_dpi_data_with_refcount!(ObjectType);
 
+// define DpiPool wrapping *mut dpiPool.
+define_dpi_data_with_refcount!(Pool);
+
 // define DpiObjectAttr wrapping *mut dpiObjectAttr.
 define_dpi_data_with_refcount!(ObjectAttr);
 
+// define DpiQueue wrapping *mut dpiQueue.
+define_dpi_data_with_refcount!(Queue);
+
 //
 // Context
 //
@@ -434,6 +447,7 @@ impl Context {
             params
         }
     }
+
     pub fn conn_create_params(&self) -> dpiConnCreateParams {
         let mut params = MaybeUninit::uninit();
         unsafe {
@@ -441,6 +455,30 @@ impl Context {
             params.assume_init()
         }
     }
+
+    pub fn pool_create_params(&self) -> dpiPoolCreateParams {
+        let mut params = MaybeUninit::uninit();
+        unsafe {
+            dpiContext_initPoolCreateParams(self.context, params.as_mut_ptr());
+            params.assume_init()
+        }
+    }
+}
+
+impl Default for dpiTimestamp {
+    fn default() -> dpiTimestamp {
+        dpiTimestamp {
+            year: 0,
+            month: 0,
+            day: 0,
+            hour: 0,
+            minute: 0,
+            second: 0,
+            fsecond: 0,
+            tzHourOffset: 0,
+            tzMinuteOffset: 0,
+        }
+    }
 }
 
 //
@@ -477,6 +515,17 @@ impl OdpiStr {
     pub fn to_string(&self) -> String {
         to_rust_str(self.ptr, self.len)
     }
+
+    #[cfg(feature = "aq_unstable")]
+    pub fn to_vec(&self) -> Vec<u8> {
+        if self.ptr.is_null() {
+            Vec::new()
+        } else {
+            let ptr = self.ptr as *mut u8;
+            let len = self.len as usize;
+            unsafe { Vec::from_raw_parts(ptr, len, len) }
+        }
+    }
 }
 
 fn to_rust_str(ptr: *const c_char, len: u32) -> String {
@@ -539,6 +588,14 @@ pub mod test_util {
         env_var_or("ODPIC_TEST_MAIN_PASSWORD", "welcome")
     }
 
+    pub fn edition_user() -> String {
+        env_var_or("ODPIC_TEST_EDITION_USER", "odpic_edition")
+    }
+
+    pub fn edition_password() -> String {
+        env_var_or("ODPIC_TEST_EDITION_PASSWORD", "welcome")
+    }
+
     pub fn connect_string() -> String {
         env_var_or("ODPIC_TEST_CONNECT_STRING", "localhost/orclpdb")
     }
