@@ -3,7 +3,7 @@
 // URL: https://github.com/kubo/rust-oracle
 //
 //-----------------------------------------------------------------------------
-// Copyright (c) 2017-2018 Kubo Takehiro <kubo@jiubao.org>. All rights reserved.
+// Copyright (c) 2017-2022 Kubo Takehiro <kubo@jiubao.org>. All rights reserved.
 // This program is free software: you can modify it and/or redistribute it
 // under the terms of:
 //
@@ -33,6 +33,8 @@ use crate::sql_type::FromSql;
 use crate::sql_type::OracleType;
 use crate::sql_type::RefCursor;
 use crate::sql_type::ToSql;
+#[cfg(doc)]
+use crate::sql_type::{Blob, Clob, Nclob};
 use crate::sql_value::BufferRowIndex;
 use crate::to_odpi_str;
 use crate::to_rust_str;
@@ -44,6 +46,8 @@ use crate::ResultSet;
 use crate::Row;
 use crate::RowValue;
 use crate::SqlValue;
+#[cfg(doc)]
+use std::io::Read;
 
 // https://www.oracle.com/pls/topic/lookup?ctx=dblatest&id=GUID-A251CF91-EB9F-4DBC-8BB8-FB5EA92C20DE
 const SQLFNCODE_CREATE_TYPE: u16 = 77;
@@ -80,6 +84,7 @@ pub struct StatementBuilder<'conn, 'sql> {
     query_params: QueryParams,
     scrollable: bool,
     tag: String,
+    exclude_from_cache: bool,
 }
 
 impl<'conn, 'sql> StatementBuilder<'conn, 'sql> {
@@ -90,6 +95,7 @@ impl<'conn, 'sql> StatementBuilder<'conn, 'sql> {
             query_params: QueryParams::new(),
             scrollable: false,
             tag: "".into(),
+            exclude_from_cache: false,
         }
     }
 
@@ -135,6 +141,42 @@ impl<'conn, 'sql> StatementBuilder<'conn, 'sql> {
         self
     }
 
+    /// Enables lob data types to be fetched or bound as [`Clob`], [`Nclob`] or [`Blob`].
+    ///
+    /// Lob data types are internally bound as string or bytes by default.
+    /// It is proper for small data but not for big data. That's because
+    /// when a lob contains 1 gigabyte data, the whole data are copied to the client
+    /// and consume 1 gigabyte or more memory. When `lob_locator` is set and
+    /// a column is fetched as [`Clob`], data are copied using [`Read::read`].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use oracle::Connection;
+    /// # use oracle::Error;
+    /// # use oracle::sql_type::Clob;
+    /// # use oracle::test_util;
+    /// # use std::io::{Read, Write};
+    /// # let conn = test_util::connect()?;
+    /// # conn.execute("delete from TestClobs", &[])?;
+    /// # conn.execute("insert into TestClobs values (:1, :2)", &[&1i32, &"clob data"])?;
+    /// # let mut out = vec![0u8; 0];
+    /// let mut stmt = conn.statement("select ClobCol from TestClobs where IntCol = :1")
+    ///     .lob_locator()
+    ///     .build()?;
+    /// let mut clob = stmt.query_row_as::<Clob>(&[&1i32])?;
+    ///
+    /// // Copy contents of clob using 1MB buffer.
+    /// let mut buf = [0u8; 1 * 1024 * 1024];
+    /// loop {
+    ///   let size = clob.read(&mut buf)?;
+    ///   if size == 0 {
+    ///     break;
+    ///   }
+    ///   out.write(&buf[0..size]);
+    /// }
+    /// # Ok::<(), Box::<dyn std::error::Error>>(())
+    /// ```
     pub fn lob_locator<'a>(&'a mut self) -> &'a mut StatementBuilder<'conn, 'sql> {
         self.query_params.lob_bind_type = LobBindType::Locator;
         self
@@ -146,8 +188,59 @@ impl<'conn, 'sql> StatementBuilder<'conn, 'sql> {
         self
     }
 
-    // make the visibility public when statement caching is supported.
-    fn tag<'a, T>(&'a mut self, tag_name: T) -> &'a mut StatementBuilder<'conn, 'sql>
+    /// Specifies the key to be used for searching for the statement in the statement cache.
+    /// If the key is not found, the SQL text specified by [`Connection::statement`] is used
+    /// to create a statement.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use oracle::Error;
+    /// # use oracle::test_util;
+    /// # let conn = test_util::connect()?;
+    ///
+    /// // When both SQL text and a tag are specifed and the tag is not found
+    /// // in the statement cache, the SQL text is used to make a statement.
+    /// // The statement is backed to the cache with the tag when
+    /// // it is closed.
+    /// let mut stmt = conn.statement("select 1 from dual").tag("query one").build()?;
+    /// assert_eq!(stmt.query_row_as::<i32>(&[])?, 1);
+    /// stmt.close()?;
+    ///
+    /// // When only a tag is specified and the tag is found in the cache,
+    /// // the statement with the tag is returned.
+    /// let mut stmt = conn.statement("").tag("query one").build()?;
+    /// assert_eq!(stmt.query_row_as::<i32>(&[])?, 1);
+    /// stmt.close()?;
+    ///
+    /// // When only a tag is specified and the tag isn't found in the cache,
+    /// // ORA-24431 is returned.
+    /// let err = conn.statement("").tag("not existing tag").build().unwrap_err();
+    /// match err {
+    ///   Error::OciError(err) if err.code() == 24431 => {
+    ///     // ORA-24431: Statement does not exist in the cache
+    ///   },
+    ///   _ => panic!("unexpected err {:?}", err),
+    /// }
+    ///
+    /// // WARNING: The SQL statement is not checked when the tag is found.
+    /// let mut stmt = conn.statement("select 2 from dual").tag("query one").build()?;
+    /// // The result must be 2 if the SQL text is used. However it is 1
+    /// // because the statement tagged with "query one" is "select 1 from dual".
+    /// assert_eq!(stmt.query_row_as::<i32>(&[])?, 1);
+    /// stmt.close()?;
+    ///
+    /// # // test whether the statement is tagged when it is closed by drop.
+    /// # {
+    /// #    let mut stmt = conn.statement("select 2 from dual").tag("query two").build()?;
+    /// #    assert_eq!(stmt.query_row_as::<i32>(&[])?, 2);
+    /// #    // stmt is dropped here.
+    /// # }
+    /// # let mut stmt = conn.statement("").tag("query two").build()?;
+    /// # assert_eq!(stmt.query_row_as::<i32>(&[])?, 2);
+    /// # Ok::<(), Error>(())
+    /// ```
+    pub fn tag<'a, T>(&'a mut self, tag_name: T) -> &'a mut StatementBuilder<'conn, 'sql>
     where
         T: Into<String>,
     {
@@ -155,6 +248,12 @@ impl<'conn, 'sql> StatementBuilder<'conn, 'sql> {
         self
     }
 
+    /// Excludes the statement from the cache even when stmt_cache_size is not zero.
+    pub fn exclude_from_cache<'a>(&'a mut self) -> &'a mut StatementBuilder<'conn, 'sql> {
+        self.exclude_from_cache = true;
+        self
+    }
+
     pub fn build(&self) -> Result<Statement<'conn>> {
         Statement::new(self)
     }
@@ -179,7 +278,7 @@ pub enum StmtParam {
     /// `StmtParam::FetchArraySize(1)`.
     FetchArraySize(u32),
 
-    /// Reserved for when statement caching is supported.
+    /// See [`StatementBuilder::tag`].
     Tag(String),
 
     /// Reserved for when scrollable cursors are supported.
@@ -293,10 +392,16 @@ pub(crate) struct Stmt {
     pub(crate) row: Option<Row>,
     shared_buffer_row_index: Rc<RefCell<u32>>,
     pub(crate) query_params: QueryParams,
+    tag: String,
 }
 
 impl Stmt {
-    pub(crate) fn new(conn: Conn, handle: *mut dpiStmt, query_params: QueryParams) -> Stmt {
+    pub(crate) fn new(
+        conn: Conn,
+        handle: *mut dpiStmt,
+        query_params: QueryParams,
+        tag: String,
+    ) -> Stmt {
         Stmt {
             conn: conn,
             handle: handle,
@@ -304,6 +409,7 @@ impl Stmt {
             row: None,
             shared_buffer_row_index: Rc::new(RefCell::new(0)),
             query_params: query_params,
+            tag: tag,
         }
     }
 
@@ -320,14 +426,8 @@ impl Stmt {
     }
 
     fn close(&mut self) -> Result<()> {
-        self.close_internal("")
-    }
-
-    fn close_internal(&mut self, tag: &str) -> Result<()> {
-        let tag = to_odpi_str(tag);
-
+        let tag = to_odpi_str(&self.tag);
         chkerr!(self.ctxt(), dpiStmt_close(self.handle, tag.ptr, tag.len));
-        self.handle = ptr::null_mut();
         Ok(())
     }
 
@@ -394,6 +494,7 @@ impl Stmt {
 
 impl Drop for Stmt {
     fn drop(&mut self) {
+        let _ = self.close();
         unsafe { dpiStmt_release(self.handle) };
     }
 }
@@ -490,8 +591,16 @@ impl<'conn> Statement<'conn> {
                 ));
             }
         };
+        let tag = if builder.exclude_from_cache {
+            chkerr!(conn.ctxt(), dpiStmt_deleteFromCache(handle), unsafe {
+                dpiStmt_release(handle);
+            });
+            String::new()
+        } else {
+            builder.tag.clone()
+        };
         Ok(Statement {
-            stmt: Stmt::new(conn.conn.clone(), handle, builder.query_params.clone()),
+            stmt: Stmt::new(conn.conn.clone(), handle, builder.query_params.clone(), tag),
             statement_type: StatementType::from_enum(info.statementType),
             is_returning: info.isReturning != 0,
             bind_count: bind_count,
@@ -551,6 +660,21 @@ impl<'conn> Statement<'conn> {
         Ok(ResultSet::new(&self.stmt))
     }
 
+    /// Executes the prepared statement and returns a result set containing [`RowValue`]s.
+    ///
+    /// This is the same as [`Statement::query_as()`], but takes ownership of the [`Statement`].
+    ///
+    /// See [Query Methods][].
+    ///
+    /// [Query Methods]: https://github.com/kubo/rust-oracle/blob/master/docs/query-methods.md
+    pub fn into_result_set<'a, T>(mut self, params: &[&dyn ToSql]) -> Result<ResultSet<'a, T>>
+    where
+        T: RowValue,
+    {
+        self.exec(params, true, "into_result_set")?;
+        Ok(ResultSet::from_stmt(self.stmt))
+    }
+
     /// Executes the prepared statement using named parameters and returns a result set containing [`RowValue`]s.
     ///
     /// See [Query Methods][].
@@ -567,6 +691,24 @@ impl<'conn> Statement<'conn> {
         Ok(ResultSet::new(&self.stmt))
     }
 
+    /// Executes the prepared statement using named parameters and returns a result set containing [`RowValue`]s.
+    ///
+    /// This is the same as [`Statement::query_as_named()`], but takes ownership of the [`Statement`].
+    ///
+    /// See [Query Methods][].
+    ///
+    /// [Query Methods]: https://github.com/kubo/rust-oracle/blob/master/docs/query-methods.md
+    pub fn into_result_set_named<'a, T>(
+        mut self,
+        params: &[(&str, &dyn ToSql)],
+    ) -> Result<ResultSet<'a, T>>
+    where
+        T: RowValue,
+    {
+        self.exec_named(params, true, "into_result_set_named")?;
+        Ok(ResultSet::from_stmt(self.stmt))
+    }
+
     /// Gets one row from the prepared statement using positoinal bind parameters.
     ///
     /// See [Query Methods][].
@@ -932,6 +1074,34 @@ impl<'conn> Statement<'conn> {
 
     /// Returns the number of rows fetched when the SQL statement is a query.
     /// Otherwise, the number of rows affected.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use oracle::Error;
+    /// # use oracle::test_util::{self, check_version, VER12_1};
+    /// # let conn = test_util::connect()?;
+    /// // number of affected rows
+    /// let stmt = conn.execute("update TestStrings set StringCol = StringCol where IntCol >= :1", &[&6])?;
+    /// assert_eq!(stmt.row_count()?, 5);
+    ///
+    /// // number of fetched rows
+    /// let mut stmt = conn.statement("select * from TestStrings where IntCol >= :1").build()?;
+    /// assert_eq!(stmt.row_count()?, 0); // before fetch
+    /// let mut nrows = 0;
+    /// for _ in stmt.query(&[&6])? {
+    ///   nrows += 1;
+    /// }
+    /// assert_eq!(stmt.row_count()?, nrows); // after fetch
+    ///
+    /// // fetch again using same stmt with a different bind value.
+    /// let mut nrows = 0;
+    /// for _ in stmt.query(&[&4])? {
+    ///   nrows += 1;
+    /// }
+    /// assert_eq!(stmt.row_count()?, nrows); // after fetch
+    /// # Ok::<(), Error>(())
+    /// ```
     pub fn row_count(&self) -> Result<u64> {
         self.stmt.row_count()
     }
